#include <pro.h>
#include <ua.hpp>

#include "base/SMPDataFlowAnalysis.h"
#include "base/SMPStaticAnalyzer.h"
#include "interfaces/SMPDBInterface.h"
#include "interfaces/abstract/all.h"
#include "interfaces/idapro/all.h"

// Clone method (deep copy).
STARSOpndTypePtr STARS_IDA_op_t::clone(void) const {
	auto TempOpPtr(std::make_shared<STARS_IDA_op_t>(this->m_Opnd));
	return std::dynamic_pointer_cast<STARS_op_t>(TempOpPtr);
} // end of STARS_IDA_op_t::clone()

// Less-than operator for use in STL ordered containers, e.g. sets.
bool STARS_IDA_op_t::operator<(const STARS_op_t &rOp) const {
	unsigned char Type1 = this->GetType();
	unsigned char Type2 = rOp.GetType();
	if (Type1 != Type2)
		return (Type1 < Type2);
	switch (Type1) {
		case o_void: return false;
		case o_reg: return MDLessReg(this->GetReg(), rOp.GetReg());
		case o_mem: return (this->GetAddr() < rOp.GetAddr());
		case o_phrase: if (this->HasSIBByte() && rOp.HasSIBByte()) return ((this->GetSIB() < rOp.GetSIB()) || ((this->GetSIB() == rOp.GetSIB()) && (this->GetSpecFlag4() < rOp.GetSpecFlag4())));
						else if (rOp.HasSIBByte()) return true;  // no SIB < has SIB
						else if (this->HasSIBByte()) return false; // no SIB < has SIB
						else return MDLessReg(this->GetReg(), rOp.GetReg()); // no SIB bytes
		case o_displ: if (this->HasSIBByte() && rOp.HasSIBByte())
						  return ((this->GetSIB() < rOp.GetSIB()) 
							|| ((this->GetSIB() == rOp.GetSIB()) 
								&& ((this->GetAddr() < rOp.GetAddr()) || ((this->GetAddr() == rOp.GetAddr()) && (this->GetSpecFlag4() < rOp.GetSpecFlag4())))));
						else if (rOp.HasSIBByte()) return true;  // no SIB < has SIB
						else if (this->HasSIBByte()) return false; // no SIB < has SIB
						else return ((this->GetAddr() < rOp.GetAddr())
							|| ((this->GetAddr() == rOp.GetAddr()) && MDLessReg(this->GetReg(), rOp.GetReg()))); // no SIB bytes
		case o_imm: return (this->GetImmedValue() < rOp.GetImmedValue());
		case o_far:  // fall through to o_near case
		case o_near: return (this->GetAddr() < rOp.GetAddr());
		case o_trreg:  // fall through
		case o_dbreg:  // fall through
		case o_crreg:  // fall through
		case o_fpreg:  // fall through
		case o_mmxreg: // fall through
		case o_xmmreg: return (this->GetReg() < rOp.GetReg()); // no subword regs to deal with, don't need MDLessReg()

		default: msg("ERROR: Unknown operand type in LessOp.\n"); return false;
	}; // end switch (Opnd1.type)
}; // end of operator less-than
		
